/***********************************************************************/
/*                                                                     */
/*                           Objective Caml                            */
/*                                                                     */
/*             Damien Doligez, projet Para, INRIA Rocquencourt         */
/*                                                                     */
/*  Copyright 1996 Institut National de Recherche en Informatique et   */
/*  Automatique.  Distributed only by permission.                      */
/*                                                                     */
/***********************************************************************/

/* $Id: major_gc.h,v 1.3 1997/09/09 18:26:45 blanchet Exp $ */

#ifndef _major_gc_
#define _major_gc_


#include "freelist.h"
#include "misc.h"

typedef struct {
  asize_t size;
  char *next;
} heap_chunk_head;

extern int gc_phase;
extern unsigned long allocated_words;
extern unsigned long extra_heap_memory;

#define Phase_mark 0
#define Phase_sweep 1

#ifdef __alpha
typedef int page_table_entry;
#else
typedef char page_table_entry;
#endif

extern char *heap_start;
extern char *heap_end;
extern unsigned long total_heap_size;
extern page_table_entry *page_table;
extern asize_t page_table_size;
extern char *gc_sweep_hp;

#define In_heap 1
#define Not_in_heap 0
#define Page(p) (((addr) (p) - (addr) heap_start) >> Page_log)
#define Is_in_heap(p) \
  ((addr)(p) >= (addr)heap_start && (addr)(p) < (addr)heap_end \
   && page_table [Page (p)])

#ifdef NATIVE_CODE
extern char * caml_top_of_stack;
#ifdef __GNUC__
#ifdef __mips__
#define SP "$sp"
#endif
#ifdef __sparc__
#define SP "%sp"
#endif
#ifdef __i386__
#define SP "%esp"
#endif
#ifdef __alpha__
#define SP "$30"
#endif
#endif

#ifdef SP
register char * stackpointer asm(SP);
#else
extern char * stack_pointer();
#define stackpointer stack_pointer()
#endif

#ifdef __sparc__
#define Is_in_stack(p) \
  ((addr)(p) >= (addr)stackpointer && (addr)(p) < (addr)caml_top_of_stack + 96)
#else
#ifndef Stack_grows_upwards
#define Is_in_stack(p) \
  ((addr)(p) >= (addr)stackpointer && (addr)(p) < (addr)caml_top_of_stack)
#else
#define Is_in_stack(p) \
  ((addr)(p) <= (addr)stackpointer && (addr)(p) > (addr)caml_top_of_stack)
#endif
#endif

#else

#define Is_in_stack(p) 0

#endif

void init_major_heap P((asize_t));
asize_t round_heap_chunk_size P((asize_t));
void darken P((value));
void major_collection_slice P((void));
void major_collection P((void));
void finish_major_cycle P((void));


#endif /* _major_gc_ */
