(***********************************************************************)
(*                                                                     *)
(*              Fast Escape Analysis - Equations solver                *)
(*                                                                     *)
(*                  Bruno Blanchet, ENS Ulm - INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

val top : int

module KnuthEquations :
  sig
    type term =
      | Term of int
      | Nonterm of nonterm
      | Min of term * term
      | Max of term * term
      | Seuil of term * term * term
      | Fun of term * (int -> int)
    and equation = { lhs: nonterm; mutable rhs: term }
    and data = equation list
    and t =
      { mutable key: int;
        mutable index: int;
        mutable l_eqlist: data;
        mutable r_eqlist: data;
        name: string }
    and nonterm = t
  end

module KnuthSolve :
  sig
    val reset : unit -> unit
    val newnonterm : string -> KnuthEquations.nonterm
    val compute : KnuthEquations.term -> int
    val addeq : KnuthEquations.nonterm -> KnuthEquations.term -> unit
    val reduce : KnuthEquations.term -> KnuthEquations.term
    val termmin :
      KnuthEquations.term -> KnuthEquations.term -> KnuthEquations.term
    val termmax :
      KnuthEquations.term -> KnuthEquations.term -> KnuthEquations.term
    val nonknuthfun :
      KnuthEquations.term -> (int -> int) -> int -> KnuthEquations.term
    val knuthfun : KnuthEquations.term -> (int -> int) -> KnuthEquations.term
    val print_term : KnuthEquations.term -> unit
    val cfc : unit -> unit
  end
